﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Proxies;
using System.Text;
using System.Threading.Tasks;

namespace Repository
{
    public class DynamicProxy<T> : RealProxy
    {

        private readonly T _decorated;

        public DynamicProxy(T decorated) : base(typeof (T))
        {
            _decorated = decorated;
        }

        private void Log(string msg, object args = null)
        {
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine(msg, args);
            Console.ResetColor();
        }

        public override IMessage Invoke(IMessage msg)
        {
            var method = msg as IMethodCallMessage;
            var methodInfo = method.MethodBase as MethodInfo;
            Log("Dynamic Proxy - Avant exécution de {0}",method.MethodName);
            try
            {
                var result = methodInfo.Invoke(_decorated, method.InArgs);
                Log("Dynamic Proxy - Après exécution de {0}", method.MethodName);
                return new ReturnMessage(result,null,0,method.LogicalCallContext,method);
            }
            catch (Exception e)
            {
                Log(string.Format("Dynamic Proxy Error {0} in {1}", e, method.MethodName));
                return new ReturnMessage(e,method);
            }
        }
    }
}
